@@ -25,8 +25,6 @@
 #include <time.h>
 
 #include <glib.h>
-#include <libgnomevfs/gnome-vfs-types.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
 
 #include "file-data.h"
 #include "file-utils.h"
@@ -34,13 +32,13 @@
 #include "fr-command.h"
 #include "fr-command-rar.h"
 
-static void fr_command_rar_class_init  (FRCommandRarClass *class);
-static void fr_command_rar_init        (FRCommand         *afile);
+static void fr_command_rar_class_init  (FrCommandRarClass *class);
+static void fr_command_rar_init        (FrCommand         *afile);
 static void fr_command_rar_finalize    (GObject           *object);
 
 /* Parent Class */
 
-static FRCommandClass *parent_class = NULL;
+static FrCommandClass *parent_class = NULL;
 
 
 static gboolean
@@ -54,12 +52,14 @@ have_rar (void)
 
 
 static time_t
-mktime_from_string (char *date_s, 
+mktime_from_string (char *date_s,
 		    char *time_s)
 {
 	struct tm   tm = {0, };
 	char      **fields;
 
+	tm.tm_isdst = -1;
+
 	/* date */
 
 	fields = g_strsplit (date_s, "-", 3);
@@ -77,8 +77,8 @@ mktime_from_string (char *date_s,
 
 	fields = g_strsplit (time_s, ":", 2);
 	if (fields[0] != NULL) {
-		tm.tm_hour = atoi (fields[0]) - 1;
-		if (fields[1] != NULL) 
+		tm.tm_hour = atoi (fields[0]);
+		if (fields[1] != NULL)
 			tm.tm_min = atoi (fields[1]);
 	}
 	g_strfreev (fields);
@@ -88,14 +88,13 @@ mktime_from_string (char *date_s,
 
 
 static void
-process_line (char     *line, 
+process_line (char     *line,
 	      gpointer  data)
 {
-	FRCommand     *comm = FR_COMMAND (data);
-	FRCommandRar  *rar_comm = FR_COMMAND_RAR (comm);
+	FrCommand     *comm = FR_COMMAND (data);
+	FrCommandRar  *rar_comm = FR_COMMAND_RAR (comm);
 	char         **fields;
 	const char    *name_field;
-	gboolean       encrypted;      /* unused */
 
 	g_return_if_fail (line != NULL);
 
@@ -118,54 +117,55 @@ process_line (char     *line,
 		rar_comm->fdata = fdata = file_data_new ();
 
 		/* read file name. */
-	  
-		encrypted = (line[0] == '*') ? TRUE : FALSE;
-		
+
+		fdata->encrypted = (line[0] == '*') ? TRUE : FALSE;
+
 		name_field = line + 1;
 
 		if (*name_field == '/') {
 			fdata->full_path = g_strdup (name_field);
 			fdata->original_path = fdata->full_path;
-		} else {
+		} 
+		else {
 			fdata->full_path = g_strconcat ("/", name_field, NULL);
 			fdata->original_path = fdata->full_path + 1;
 		}
-		
+
 		fdata->link = NULL;
 		fdata->path = remove_level_from_path (fdata->full_path);
-
-	} else {
+	} 
+	else {
 		FileData *fdata;
-		
+
 		fdata = rar_comm->fdata;
 
 		/* read file info. */
-		
+
 		fields = split_line (line, 6);
 
 		fdata->size = g_ascii_strtoull (fields[0], NULL, 10);
 		fdata->modified = mktime_from_string (fields[3], fields[4]);
 
 		if ((fields[5][1] == 'D') || (fields[5][0] == 'd')) {
-			gboolean  original_is_full = fdata->original_path == fdata->full_path;
-			char     *path = fdata->full_path;
-						
-			fdata->full_path = g_strconcat (path, "/", NULL);
-			if (original_is_full)
-				fdata->original_path = fdata->full_path;
-			else
-				fdata->original_path = fdata->full_path + 1;
+			char *tmp;
+			
+			tmp = fdata->full_path;
+			fdata->full_path = g_strconcat (fdata->full_path, "/", NULL);
+			
+			fdata->original_path = g_strdup (fdata->original_path);
+			fdata->free_original_path = TRUE;
+			
+			g_free (tmp);
+			
 			fdata->name =  dir_name_from_path (fdata->full_path);
-			
 			fdata->dir = TRUE;
-			
-			g_free (path);
-		} else
+		} 
+		else
 			fdata->name = g_strdup (file_name_from_path (fdata->full_path));
 
-		comm->file_list = g_list_prepend (comm->file_list, fdata);
+		fr_command_add_file (comm, fdata);
 		rar_comm->fdata = NULL;
-		
+
 		g_strfreev (fields);
 	}
 
@@ -174,34 +174,34 @@ process_line (char     *line,
 
 
 static void
-add_password_arg (FRCommand	*comm,
+add_password_arg (FrCommand	*comm,
 		  const char	*password,
 		  gboolean	disable_query)
 {
 	if (password != NULL && password[0] != '\0') {
 		char *arg, *e_password;
-		
+
 		e_password = escape_str (password, "\"*?[]'`()$!;");
 		g_assert (e_password != NULL);  /* since password is non-NULL */
-		
+
 		arg = g_strconcat ("-p\"", e_password, "\"", NULL);
-		
+
 		fr_process_add_arg (comm->process, arg);
-		
+
 		g_free (e_password);
 		g_free (arg);
-	} else if (disable_query) 
+	} else if (disable_query)
 		fr_process_add_arg (comm->process, "-p-");
 }
 
 
 static void
-fr_command_rar_list (FRCommand  *comm,
+fr_command_rar_list (FrCommand  *comm,
 		     const char *password)
 {
 	FR_COMMAND_RAR (comm)->list_started = FALSE;
 
-	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
+	fr_process_set_out_line_func (FR_COMMAND (comm)->process,
 				      process_line,
 				      comm);
 
@@ -213,9 +213,9 @@ fr_command_rar_list (FRCommand  *comm,
 	fr_process_add_arg (comm->process, "-c-");
 
 	add_password_arg (comm, password, TRUE);
-	
+
 	/* stop switches scanning */
-	fr_process_add_arg (comm->process, "--"); 
+	fr_process_add_arg (comm->process, "--");
 
 	fr_process_add_arg (comm->process, comm->e_filename);
 	fr_process_end_command (comm->process);
@@ -224,7 +224,7 @@ fr_command_rar_list (FRCommand  *comm,
 
 
 static void
-fr_command_rar_add (FRCommand     *comm,
+fr_command_rar_add (FrCommand     *comm,
 		    GList         *file_list,
 		    const char    *base_dir,
 		    gboolean       update,
@@ -235,7 +235,7 @@ fr_command_rar_add (FRCommand     *comm,
 
 	fr_process_begin_command (comm->process, "rar");
 
-	if (base_dir != NULL) 
+	if (base_dir != NULL)
 		fr_process_set_working_dir (comm->process, base_dir);
 
 	if (update)
@@ -257,14 +257,14 @@ fr_command_rar_add (FRCommand     *comm,
 	add_password_arg (comm, password, FALSE);
 
 	/* disable percentage indicator */
-	fr_process_add_arg (comm->process, "-idp");
-	
+	fr_process_add_arg (comm->process, "-Idp");
+
 	/* stop switches scanning */
-	fr_process_add_arg (comm->process, "--"); 
+	fr_process_add_arg (comm->process, "--");
 
 	fr_process_add_arg (comm->process, comm->e_filename);
 
-	for (scan = file_list; scan; scan = scan->next) 
+	for (scan = file_list; scan; scan = scan->next)
 		fr_process_add_arg (comm->process, (gchar*) scan->data);
 
 	fr_process_end_command (comm->process);
@@ -272,7 +272,7 @@ fr_command_rar_add (FRCommand     *comm,
 
 
 static void
-fr_command_rar_delete (FRCommand *comm,
+fr_command_rar_delete (FrCommand *comm,
 		       GList     *file_list)
 {
 	GList *scan;
@@ -281,7 +281,7 @@ fr_command_rar_delete (FRCommand *comm,
 	fr_process_add_arg (comm->process, "d");
 
 	/* stop switches scanning */
-	fr_process_add_arg (comm->process, "--"); 
+	fr_process_add_arg (comm->process, "--");
 
 	fr_process_add_arg (comm->process, comm->e_filename);
 
@@ -292,7 +292,7 @@ fr_command_rar_delete (FRCommand *comm,
 
 
 static void
-fr_command_rar_extract (FRCommand  *comm,
+fr_command_rar_extract (FrCommand  *comm,
 			GList      *file_list,
 			const char *dest_dir,
 			gboolean    overwrite,
@@ -312,21 +312,21 @@ fr_command_rar_extract (FRCommand  *comm,
 	if (overwrite)
 		fr_process_add_arg (comm->process, "-o+");
 	else
-		fr_process_add_arg (comm->process, "-o-");	
+		fr_process_add_arg (comm->process, "-o-");
 
 	if (skip_older)
 		fr_process_add_arg (comm->process, "-u");
 
 	if (junk_paths)
 		fr_process_add_arg (comm->process, "-ep");
-	
+
 	add_password_arg (comm, password, TRUE);
 
 	/* disable percentage indicator */
-	fr_process_add_arg (comm->process, "-idp");
-	
+	fr_process_add_arg (comm->process, "-Idp");
+
 	/* stop switches scanning */
-	fr_process_add_arg (comm->process, "--"); 
+	fr_process_add_arg (comm->process, "--");
 
 	fr_process_add_arg (comm->process, comm->e_filename);
 
@@ -338,61 +338,72 @@ fr_command_rar_extract (FRCommand  *comm,
 		fr_process_add_arg (comm->process, e_dest_dir);
 		g_free (e_dest_dir);
 	}
-	
+
 	fr_process_end_command (comm->process);
 }
 
 
 static void
-fr_command_rar_test (FRCommand   *comm,
+fr_command_rar_test (FrCommand   *comm,
 		     const char  *password)
 {
 	if (have_rar ())
 		fr_process_begin_command (comm->process, "rar");
 	else
 		fr_process_begin_command (comm->process, "unrar");
-	
+
 	fr_process_add_arg (comm->process, "t");
-	
+
 	add_password_arg (comm, password, TRUE);
 
 	/* disable percentage indicator */
-	fr_process_add_arg (comm->process, "-idp");
+	fr_process_add_arg (comm->process, "-Idp");
 
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
-	
+
 	fr_process_add_arg (comm->process, comm->e_filename);
 	fr_process_end_command (comm->process);
 }
 
 
 static void
-fr_command_rar_handle_error (FRCommand   *comm, 
+fr_command_rar_handle_error (FrCommand   *comm,
 			     FRProcError *error)
 {
-	if (error->type == FR_PROC_ERROR_GENERIC) {
+	if (error->type == FR_PROC_ERROR_COMMAND_ERROR) {
 		if (error->status <= 1)
 			error->type = FR_PROC_ERROR_NONE;
 		else if (error->status == 3)
 			error->type = FR_PROC_ERROR_ASK_PASSWORD;
-	} else if (error->type == FR_PROC_ERROR_EXITED_ABNORMALLY)
-		error->type = FR_PROC_ERROR_ASK_PASSWORD;
+		else {
+			GList *scan;
+			
+			for (scan = g_list_last (comm->process->raw_error); scan; scan = scan->prev) {
+				char *line = scan->data;
+				
+				if (strstr (line, "password incorrect") != NULL) {
+					error->type = FR_PROC_ERROR_ASK_PASSWORD;
+					break;
+				}
+			}
+		}
+	}
 }
 
 
-static void 
-fr_command_rar_class_init (FRCommandRarClass *class)
+static void
+fr_command_rar_class_init (FrCommandRarClass *class)
 {
-        GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-        FRCommandClass *afc;
+	GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+	FrCommandClass *afc;
 
-        parent_class = g_type_class_peek_parent (class);
-	afc = (FRCommandClass*) class;
+	parent_class = g_type_class_peek_parent (class);
+	afc = (FrCommandClass*) class;
 
 	gobject_class->finalize = fr_command_rar_finalize;
 
-        afc->list         = fr_command_rar_list;
+	afc->list         = fr_command_rar_list;
 	afc->add          = fr_command_rar_add;
 	afc->delete       = fr_command_rar_delete;
 	afc->extract      = fr_command_rar_extract;
@@ -400,14 +411,14 @@ fr_command_rar_class_init (FRCommandRarClass *class)
 	afc->handle_error = fr_command_rar_handle_error;
 }
 
- 
-static void 
-fr_command_rar_init (FRCommand *comm)
+
+static void
+fr_command_rar_init (FrCommand *comm)
 {
 	comm->file_type = FR_FILE_TYPE_RAR;
 
-	comm->propAddCanUpdate             = TRUE; 
-	comm->propAddCanReplace            = TRUE; 
+	comm->propAddCanUpdate             = TRUE;
+	comm->propAddCanReplace            = TRUE;
 	comm->propAddCanStoreFolders       = TRUE;
 	comm->propExtractCanAvoidOverwrite = TRUE;
 	comm->propExtractCanSkipOlder      = TRUE;
@@ -417,14 +428,14 @@ fr_command_rar_init (FRCommand *comm)
 }
 
 
-static void 
+static void
 fr_command_rar_finalize (GObject *object)
 {
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (FR_IS_COMMAND_RAR (object));
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (FR_IS_COMMAND_RAR (object));
 
 	/* Chain up */
-        if (G_OBJECT_CLASS (parent_class)->finalize)
+	if (G_OBJECT_CLASS (parent_class)->finalize)
 		G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -432,17 +443,17 @@ fr_command_rar_finalize (GObject *object)
 GType
 fr_command_rar_get_type ()
 {
-        static GType type = 0;
+	static GType type = 0;
 
-        if (! type) {
-                GTypeInfo type_info = {
-			sizeof (FRCommandRarClass),
+	if (! type) {
+		GTypeInfo type_info = {
+			sizeof (FrCommandRarClass),
 			NULL,
 			NULL,
 			(GClassInitFunc) fr_command_rar_class_init,
 			NULL,
 			NULL,
-			sizeof (FRCommandRar),
+			sizeof (FrCommandRar),
 			0,
 			(GInstanceInitFunc) fr_command_rar_init
 		};
@@ -451,19 +462,19 @@ fr_command_rar_get_type ()
 					       "FRCommandRar",
 					       &type_info,
 					       0);
-        }
+	}
 
-        return type;
+	return type;
 }
 
 
-FRCommand *
-fr_command_rar_new (FRProcess  *process,
+FrCommand *
+fr_command_rar_new (FrProcess  *process,
 		    const char *filename)
 {
-	FRCommand *comm;
+	FrCommand *comm;
 
-	if ((!is_program_in_path("rar")) && (!is_program_in_path("unrar"))) 
+	if ((!is_program_in_path("rar")) && (!is_program_in_path("unrar")))
 		return NULL;
 
 	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_RAR, NULL));
